-- luajalayout_aux.lua 2011/03/16 v0.20
-- Copyright (c) 2010-2011 Kazuki Maeda

-- define the language number
luajalayout.lang = {}
luajalayout.lang.ot = 1
luajalayout.lang.ja = 2

-- define the language table of characters
luajalayout.lang_char_table = {}
luajalayout.lang_char_table[luajalayout.lang.ja] = {}
local ja_char_table = luajalayout.lang_char_table[luajalayout.lang.ja]
ja_char_table['General Punctuation'] = {0x2000, 0x206F}
ja_char_table['Supplemental Punctuation'] = {0x2E00, 0x2E7F}
ja_char_table['CJK Radicals Supplement'] = {0x2E80, 0x2EFF}
ja_char_table['Kangxi Radicals'] = {0x2F00, 0x2FDF}
ja_char_table['CJK Symbols and Punctuation'] = {0x3000, 0x303F}
ja_char_table['Hiragana'] = {0x3040, 0x309F}
ja_char_table['Katakana'] = {0x30A0, 0x30FF}
ja_char_table['Katakana Phonetic Extensions'] = {0x31F0, 0x31FF}
ja_char_table['Enclosed CJK Letters and Months'] = {0x3200, 0x32FF}
ja_char_table['CJK Compatibility'] = {0x3300, 0x33FF}
ja_char_table['CJK Unified Ideographs Extension A'] = {0x3400, 0x4DBF}
ja_char_table['CJK Unified Ideographs'] = {0x4E00, 0x9FFF}
ja_char_table['CJK Compatibility Ideographs'] = {0xF900, 0xFAFF}
ja_char_table['Vertical Forms'] = {0xFE10, 0xFE1F}
ja_char_table['CJK Compatibility Forms'] = {0xFE30, 0xFE4F}
ja_char_table['Halfwidth and Fullwidth Forms'] = {0xFF00, 0xFFEF}
ja_char_table['Kana Supplement'] = {0x1B000, 0x1B0FF}
ja_char_table['CJK Unified Ideographs Extension B'] = {0x20000, 0x2A6DF}
ja_char_table['CJK Unified Ideographs Extension C'] = {0x2A700, 0x2B73F}
ja_char_table['CJK Unified Ideographs Extension D'] = {0x2B740, 0x2B81F}
ja_char_table['CJK Compatibility Ideographs Supplement'] = {0x2F800, 0x2FA1F}

-- define the character class

-- common members: inhibit_ot_ja_glue, prebreakpenalty, postbreakpenalty
-- if lang == ja, additional members: ja_class, ja_width, ja_hang_punct
luajalayout.char_table = {} 

luajalayout.ja_class = {}
luajalayout.ja_class.other = 0
luajalayout.ja_class.leftparen = 1
luajalayout.ja_class.rightparen = 2
luajalayout.ja_class.centerpoint = 3
luajalayout.ja_class.period = 4
luajalayout.ja_class.leader = 5
luajalayout.ja_class.len = 5

luajalayout.ja_width = {}
luajalayout.ja_width.halfright = 1
luajalayout.ja_width.halfleft = 2
luajalayout.ja_width.halfcenter = 3

luajalayout.inhibit_ot_ja_glue = {}
luajalayout.inhibit_ot_ja_glue.both = 0
luajalayout.inhibit_ot_ja_glue.before = 1
luajalayout.inhibit_ot_ja_glue.after = 2

local function add_char_table_ja_leftparen(char)
   luajalayout.char_table[char] = {ja_class = luajalayout.ja_class.leftparen, ja_width = luajalayout.ja_width.halfright, inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.after, postbreakpenalty = 10000}
end
local ja_leftparens = {"“", "（", "〔", "［", "｛", "〈", "《", "「", "『", "【"}
for k, v in pairs(ja_leftparens) do
   add_char_table_ja_leftparen(utf.byte(v))
end

local function add_char_table_ja_rightparen(char)
   luajalayout.char_table[char] = {ja_class = luajalayout.ja_class.rightparen, ja_width = luajalayout.ja_width.halfleft, inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
end
local ja_rightparens = {"、", "，", "’", "”", "）", "〕", "］", "｝", "〉", "》",
                        "」", "』", "】", "。", "．"}
for k, v in pairs(ja_rightparens) do
   add_char_table_ja_rightparen(utf.byte(v))
end

luajalayout.char_table[utf.byte("。")].ja_class = luajalayout.ja_class.period
luajalayout.char_table[utf.byte("．")].ja_class = luajalayout.ja_class.period

local function add_char_table_ja_centerpoint(char)
   luajalayout.char_table[char] = {ja_class = luajalayout.ja_class.centerpoint, ja_width = luajalayout.ja_width.halfcenter, prebreakpenalty = 10000}
end
local ja_centerpoints = {"・", "：", "；"}
for k, v in pairs(ja_centerpoints) do
   add_char_table_ja_centerpoint(utf.byte(v))
end


luajalayout.char_table[utf.byte("―")] = {ja_class = luajalayout.ja_class.leader, inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.both}
luajalayout.char_table[utf.byte("…")] = {ja_class = luajalayout.ja_class.leader, inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.both, prebreakpenalty = 250}
luajalayout.char_table[utf.byte("‥")] = {ja_class = luajalayout.ja_class.leader, inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.both, prebreakpenalty = 250}

luajalayout.char_table[utf.byte("〜")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.both}
luajalayout.char_table[utf.byte("￥")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.both}
luajalayout.char_table[utf.byte("？")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("°")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before}
luajalayout.char_table[utf.byte("′")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before}
luajalayout.char_table[utf.byte("″")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before}
luajalayout.char_table[utf.byte("‘")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.after, postbreakpenalty = 10000}
luajalayout.char_table[utf.byte("！")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("゛")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("゜")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("´")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("々")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("’")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("ー")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("＋")] = {prebreakpenalty = 200}
luajalayout.char_table[utf.byte("−")] = {prebreakpenalty = 200}
luajalayout.char_table[utf.byte("＝")] = {prebreakpenalty = 200}

local function add_char_table_ja_small_kana(char)
   luajalayout.char_table[char] = {prebreakpenalty = 150}
end
local ja_small_kanas = {"ぁ", "ぃ", "ぅ", "ぇ", "ぉ", "ァ", "ィ", "ゥ", "ェ", "ォ",
                        "っ", "ゃ", "ゅ", "ょ", "ゎ", "ゕ", "ゖ",
                        "ッ", "ャ", "ュ", "ョ", "ヮ", "ヵ", "ヶ",
                        "ㇰ", "ㇱ", "ㇲ", "ㇳ", "ㇴ", "ㇵ", "ㇶ", "ㇷ", "ㇸ", "ㇹ",
                        "ㇺ", "ㇻ", "ㇼ", "ㇽ", "ㇾ", "ㇿ", "ㇷ゚"}
for k, v in pairs(ja_small_kanas) do
   add_char_table_ja_small_kana(utf.byte(v))
end

luajalayout.char_table[utf.byte("｀")] = {postbreakpenalty = 10000}
luajalayout.char_table[utf.byte("＃")] = {postbreakpenalty = 200}
luajalayout.char_table[utf.byte("＄")] = {postbreakpenalty = 200}
luajalayout.char_table[utf.byte("％")] = {postbreakpenalty = 200}
luajalayout.char_table[utf.byte("＆")] = {postbreakpenalty = 200}

luajalayout.char_table[utf.byte("(")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.after, postbreakpenalty = 10000}
luajalayout.char_table[utf.byte("[")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.after, postbreakpenalty = 10000}
luajalayout.char_table[utf.byte("`")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.after, postbreakpenalty = 10000}
luajalayout.char_table[utf.byte(")")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("]")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("'")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte(";")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte(":")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte(",")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte(".")] = {inhibit_ot_ja_glue = luajalayout.inhibit_ot_ja_glue.before, prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("!")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("\"")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("#")] = {postbreakpenalty = 500}
luajalayout.char_table[utf.byte("$")] = {postbreakpenalty = 500}
luajalayout.char_table[utf.byte("%")] = {postbreakpenalty = 500}
luajalayout.char_table[utf.byte("&")] = {postbreakpenalty = 500}
luajalayout.char_table[utf.byte("+")] = {prebreakpenalty = 500}
luajalayout.char_table[utf.byte("-")] = {prebreakpenalty = 10000}
luajalayout.char_table[utf.byte("*")] = {prebreakpenalty = 500}
luajalayout.char_table[utf.byte("/")] = {prebreakpenalty = 500}

-- hanging punctuation
luajalayout.char_table[utf.byte("、")].ja_hang_punct = true
luajalayout.char_table[utf.byte("，")].ja_hang_punct = true
luajalayout.char_table[utf.byte("。")].ja_hang_punct = true
luajalayout.char_table[utf.byte("．")].ja_hang_punct = true

-- define glue inserted between two glyphs (unit: em)
luajalayout.ot_ja_glue = {width = 0.25, stretch = 0.15, shrink = 0.06} 
luajalayout.ja_ja_glue_other = {stretch = 0.1, shrink = 0.1}

luajalayout.ja_ja_glue_table = {}
for i = 0, luajalayout.ja_class.len, 1 do
   luajalayout.ja_ja_glue_table[i] = {}
end

luajalayout.ja_ja_glue_table[luajalayout.ja_class.leftparen][luajalayout.ja_class.centerpoint] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.rightparen][luajalayout.ja_class.other] = {width = 0.5, shrink = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.rightparen][luajalayout.ja_class.leftparen] = {width = 0.5, shrink = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.rightparen][luajalayout.ja_class.leader] = {width = 0.5, shrink = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.rightparen][luajalayout.ja_class.centerpoint] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.other] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.leftparen] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.rightparen] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.period] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.leader] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.centerpoint][luajalayout.ja_class.centerpoint] = {width = 0.5, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.period][luajalayout.ja_class.other] = {width = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.period][luajalayout.ja_class.leftparen] = {width = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.period][luajalayout.ja_class.leader] = {width = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.period][luajalayout.ja_class.centerpoint] = {width = 0.75, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.leader][luajalayout.ja_class.leftparen] = {width = 0.5, shrink = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.leader][luajalayout.ja_class.centerpoint] = {width = 0.25, shrink = 0.25}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.leader][luajalayout.ja_class.leader] = {kern = 0}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.other][luajalayout.ja_class.leftparen] = {width = 0.5, shrink = 0.5}
luajalayout.ja_ja_glue_table[luajalayout.ja_class.other][luajalayout.ja_class.centerpoint] = {width = 0.25, shrink = 0.25}

